@@ -27,6 +27,9 @@
 #include "oaf-i18n.h"
 #include "liboaf/liboaf-private.h"
 
+#include "liboaf/oaf-mainloop.h"
+#include "liboaf/oaf-registration.h"
+
 #include <sys/socket.h>
 #include <netinet/in.h>
 #include <netdb.h>
@@ -119,23 +122,24 @@ const char *
 oaf_hostname_get (void)
 {
 	static char *hostname = NULL;
-	char hn_tmp[65];
-	struct hostent *hent, *hent2;
+	char hn_tmp[65], ha_tmp[4];
+	struct hostent *hent;
 
 	if (!hostname) {
 		gethostname (hn_tmp, sizeof (hn_tmp) - 1);
 
 		hent = gethostbyname (hn_tmp);
 		if (hent) {
-			hent2 = gethostbyaddr (hent->h_addr, 4, AF_INET);
-			if (hent2)
-				hostname = g_strdup (hent2->h_name);
+			memcpy (ha_tmp, hent->h_addr, 4);
+			hent = gethostbyaddr (ha_tmp, 4, AF_INET);
+			if (hent)
+				hostname = g_strdup (hent->h_name);
 			else
 				hostname =
 					g_strdup (inet_ntoa
 						  (*
 						   ((struct in_addr *)
-						    hent->h_addr)));
+						    ha_tmp)));
 		} else
 			hostname = g_strdup (hn_tmp);
 	}
@@ -143,6 +147,7 @@ oaf_hostname_get (void)
 	return hostname;
 }
 
+
 CORBA_Context
 oaf_context_get (void)
 {
@@ -299,7 +304,7 @@ local_activator (const OAFRegistrationCategory * regcat, const char **cmd,
 		|| STRMATCH (regcat->hostname, oaf_hostname_get ()))
 	    && (!regcat->domain
 		|| STRMATCH (regcat->domain, oaf_domain_get ()))) {
-		return oaf_server_by_forking (cmd, fd_arg, ev);
+		return oaf_server_by_forking (cmd, fd_arg, NULL, ev);
 	}
 
 	return CORBA_OBJECT_NIL;
@@ -417,6 +422,7 @@ oaf_orb_init (int *argc, char **argv)
 {
 	CORBA_Environment ev;
 	const char *hostname;
+        char *display;
 
 #ifndef ORBIT_USES_GLIB_MAIN_LOOP
 	IIOPAddConnectionHandler = orb_add_connection;
@@ -438,6 +444,14 @@ oaf_orb_init (int *argc, char **argv)
 	CORBA_Context_set_one_value (oaf_context, "domain", "user", &ev);
 	CORBA_Context_set_one_value (oaf_context, "username",
 				     g_get_user_name (), &ev);
+        
+        
+        display =  g_getenv ("DISPLAY");
+        
+        if (display != NULL) {
+                CORBA_Context_set_one_value (oaf_context, "display",
+                                             display, &ev);
+        }
 
 	CORBA_exception_free (&ev);
 
