@@ -107,9 +107,8 @@ oaf_registration_check (const OAFRegistrationCategory * regcat,
 		if (!ri->regloc->check)
 			continue;
 
-		new_ior =
-			ri->regloc->check (ri->regloc, regcat, &new_dist,
-					   ri->user_data);
+		new_ior = ri->regloc->check (ri->regloc, regcat, 
+                                             &new_dist, ri->user_data);
 		if (new_ior && (new_dist < dist)) {
 			g_free (ior);
 			ior = new_ior;
@@ -275,8 +274,26 @@ oaf_strsignal (int sig)
 }
 #endif
 
+
+static 
+void oaf_setenv (const char *name, const char *value) 
+{
+#if HAVE_SETENV
+        setenv (name, value, 1);
+#else
+        char *tmp;
+                
+        tmp = g_strconcat (name, "=", value, NULL);
+        
+        putenv (tmp);
+#endif
+}
+
 CORBA_Object
-oaf_server_by_forking (const char **cmd, int fd_arg, CORBA_Environment * ev)
+oaf_server_by_forking (const char **cmd, 
+                       int fd_arg, 
+                       const char *display, 
+                       CORBA_Environment * ev)
 {
 	gint iopipes[2];
 	CORBA_Object retval = CORBA_OBJECT_NIL;
@@ -288,13 +305,20 @@ oaf_server_by_forking (const char **cmd, int fd_arg, CORBA_Environment * ev)
         int status;
         guint watchid;
         struct sigaction sa;
+        sigset_t mask, omask;
                 
      	pipe (iopipes);
 
+        /* Block SIGCHLD so no one else can wait() on the child before us. */
+        sigemptyset (&mask);
+        sigaddset (&mask, SIGCHLD);
+        sigprocmask (SIG_BLOCK, &mask, &omask);
+
 	/* fork & get the IOR from the magic pipe */
 	childpid = fork ();
 
 	if (childpid < 0) {
+                sigprocmask (SIG_SETMASK, &omask, NULL);
 		errval = OAF_GeneralError__alloc ();
 		errval->description = CORBA_string_dup (_("Couldn't fork a new process"));
 
@@ -304,7 +328,10 @@ oaf_server_by_forking (const char **cmd, int fd_arg, CORBA_Environment * ev)
 	}
 
 	if (childpid) {
-		waitpid (childpid, &status, 0);	/* de-zombify */
+                /* de-zombify */
+                while (waitpid (childpid, &status, 0) == -1 && errno == EINTR)
+                        ;
+                sigprocmask (SIG_SETMASK, &omask, NULL);
                 
 		if (!WIFEXITED (status)) {
 			OAF_GeneralError *errval;
@@ -377,6 +404,11 @@ oaf_server_by_forking (const char **cmd, int fd_arg, CORBA_Environment * ev)
 	} else if ((childpid = fork ())) {
 		_exit (0);	/* de-zombifier process, just exit */
 	} else {
+                if (display != NULL) {
+                        oaf_setenv ("DISPLAY", display);
+                }
+                
+
 		close (iopipes[0]);
                 
                 if (fd_arg != 0) {
@@ -418,14 +450,11 @@ struct SysServer
 }
 activatable_servers[] =
 {
-	{
-		"IDL:OAF/ActivationContext:1.0", (const char **) oaf_ac_cmd,
-			2, CORBA_OBJECT_NIL}
-	, {
-		"IDL:OAF/ObjectDirectory:1.0", (const char **) oaf_od_cmd,
-			1, CORBA_OBJECT_NIL}
-	, {
-	NULL}
+	{"IDL:OAF/ActivationContext:1.0", (const char **) oaf_ac_cmd,
+         2, CORBA_OBJECT_NIL}, 
+        {"IDL:OAF/ObjectDirectory:1.0", (const char **) oaf_od_cmd,
+         1, CORBA_OBJECT_NIL},
+	{ NULL}
 };
 
 #define STRMATCH(x, y) ((!x && !y) || (x && y && !strcmp(x, y)))
@@ -446,15 +475,17 @@ existing_check (const OAFRegistrationCategory * regcat, struct SysServer *ss)
 		    && (!ssi->domain
 			|| STRMATCH (ssi->domain,
 				     regcat->
-				     domain))) return ssi->already_running;
+				     domain))) {
+                        return ssi->already_running;
+                }
 	}
 
 	return CORBA_OBJECT_NIL;
 }
 
 static void
-existing_set (const OAFRegistrationCategory * regcat, struct SysServer *ss,
-	      CORBA_Object obj, CORBA_Environment * ev)
+oaf_existing_set (const OAFRegistrationCategory * regcat, struct SysServer *ss,
+	          CORBA_Object obj, CORBA_Environment * ev)
 {
 	GSList *cur;
 	struct SysServerInstance *ssi;
@@ -584,7 +615,7 @@ oaf_service_get (const OAFRegistrationCategory * regcat)
 	oaf_reglocs_unlock (ev);
 
 	if (!CORBA_Object_non_existent (retval, ev))
-		existing_set (regcat, &activatable_servers[i], retval, ev);
+		oaf_existing_set (regcat, &activatable_servers[i], retval, ev);
 
       out:
 	CORBA_exception_free (&myev);
