@@ -11,13 +11,13 @@ orbit_libdirs=
 orbit_libs=
 for i in $orbit_libs_output; do
 	case "$i" in
-	-L*) orbit_libdirs="$orbit_libdir $i" ;;
+	-L*|-R*) orbit_libdirs="$orbit_libdirs $i" ;;
 	*) orbit_libs="$orbit_libs $i" ;;
 	esac
 done
 
-OAF_LIBDIR="-L${exec_prefix}/lib $orbit_libdir"
+OAF_LIBDIR="-L${exec_prefix}/lib $orbit_libdirs"
 OAF_LIBS="-loaf $orbit_libs"
 OAF_INCLUDEDIR="-I${prefix}/include $orbit_includes"
-MODULE_VERSION="oaf-0.5.1"
+MODULE_VERSION="oaf-0.6.0"
 
