@@ -30,6 +30,7 @@
 #include "oaf-i18n.h"
 #include "liboaf/liboaf.h"
 #include "ac-query-expr.h"
+#include "oafd-corba-extensions.h"
 
 #define OAF_LINK_TIME_TO_LIVE 256
 
@@ -54,6 +55,17 @@ typedef struct
 }
 ChildODInfo;
 
+
+
+/* forward declarations */
+char     *ac_aid_to_query_string       (OAF_ActivationID aid);
+void      ac_context_to_string_array   (CORBA_Context context, 
+                                        char *sort_criteria[4],
+                                        CORBA_Environment *ev);
+
+
+
+
 static ChildODInfo *
 child_od_info_new (OAF_ObjectDirectory obj, CORBA_Environment * ev)
 {
@@ -184,6 +196,19 @@ child_od_info_free (ChildODInfo * child, CORBA_Environment * ev)
 	g_free (child);
 }
 
+
+static char *
+ac_CORBA_Context_get_value (CORBA_Context         ctx, 
+                            const char           *propname,
+                            CORBA_Environment    *ev)
+{
+        return oafd_CORBA_Context_get_value (ctx, 
+                                             propname,
+                                             ex_OAF_ActivationContext_IncompleteContext,
+                                             ev);
+}
+
+
 /*** App-specific servant structures ***/
 
 typedef struct
@@ -226,6 +251,15 @@ impl_OAF_ActivationContext_activate (impl_POA_OAF_ActivationContext * servant,
 				     CORBA_Context ctx,
 				     CORBA_Environment * ev);
 
+static void
+impl_OAF_ActivationContext_activate_async(impl_POA_OAF_ActivationContext *
+					  servant, CORBA_char * requirements,
+					  GNOME_stringlist * selection_order,
+					  OAF_ActivationFlags flags,
+					  OAF_ActivationCallback
+					  callback_object, CORBA_Context ctx,
+					  CORBA_Environment * ev);
+
 static OAF_ServerInfoList
 	* impl_OAF_ActivationContext__get_servers
 	(impl_POA_OAF_ActivationContext * servant, CORBA_Environment * ev);
@@ -245,6 +279,13 @@ static OAF_ActivationResult
 	 OAF_ActivationFlags flags, CORBA_Context ctx,
 	 CORBA_Environment * ev);
 
+static void
+impl_OAF_ActivationContext_activate_from_id_async
+   (impl_POA_OAF_ActivationContext * servant, OAF_ActivationID aid,
+    OAF_ActivationFlags flags, OAF_ActivationCallback callback_object,
+    CORBA_Context ctx, CORBA_Environment * ev);
+
+
 static void
 ac_query_run (impl_POA_OAF_ActivationContext * servant,
 	      CORBA_char * requirements,
@@ -272,9 +313,11 @@ static POA_OAF_ActivationContext__epv impl_OAF_ActivationContext_epv = {
 	(gpointer) &impl_OAF_ActivationContext_add_directory,
 	(gpointer) &impl_OAF_ActivationContext_remove_directory,
 	(gpointer) &impl_OAF_ActivationContext_activate,
+        (gpointer) &impl_OAF_ActivationContext_activate_async,
 	(gpointer) &impl_OAF_ActivationContext__get_servers,
 	(gpointer) &impl_OAF_ActivationContext_query,
-	(gpointer) &impl_OAF_ActivationContext_activate_from_id
+	(gpointer) &impl_OAF_ActivationContext_activate_from_id,
+        (gpointer) &impl_OAF_ActivationContext_activate_from_id_async
 };
 
 /*** vepv structures ***/
@@ -521,40 +564,6 @@ ac_do_activation (impl_POA_OAF_ActivationContext * servant,
 	}
 }
 
-static char *
-ctx_get_value (CORBA_Context ctx, const char *propname,
-	       CORBA_Environment * ev)
-{
-	CORBA_NVList *nvout;
-	char *retval;
-
-	/* Figure out what the host the 
-         * activating client is running on (for
-	 * purposes of fun & pleasure) 
-         */
-	CORBA_Context_get_values (ctx, NULL, 0, (char *) propname, &nvout,
-				  ev);
-	if (ev->_major == CORBA_NO_EXCEPTION) {
-		if (nvout->list->len > 0) {
-			CORBA_NamedValue *nv;
-
-			nv = &g_array_index (nvout->list, CORBA_NamedValue, 0);
-			retval = g_strdup (*(char **) nv->argument._value);
-		} else
-			retval = NULL;
-
-		CORBA_NVList_free (nvout, ev);
-	} else
-        {                
-                CORBA_exception_set (ev, CORBA_USER_EXCEPTION,
-                                     ex_OAF_ActivationContext_IncompleteContext,
-                                     NULL);
-                
-		retval = NULL;
-        }
-
-	return retval;
-}
 
 static OAF_ActivationResult *
 impl_OAF_ActivationContext_activate (impl_POA_OAF_ActivationContext * servant,
@@ -569,7 +578,7 @@ impl_OAF_ActivationContext_activate (impl_POA_OAF_ActivationContext * servant,
 	int i;
 	char *hostname;
 
-	hostname = ctx_get_value (ctx, "hostname", ev);
+	hostname = ac_CORBA_Context_get_value (ctx, "hostname", ev);
 	ac_update_lists (servant, ev);
 
 	servant->refs++;
@@ -604,6 +613,75 @@ impl_OAF_ActivationContext_activate (impl_POA_OAF_ActivationContext * servant,
 	return retval;
 }
 
+
+static void
+impl_OAF_ActivationContext_activate_async(impl_POA_OAF_ActivationContext *
+					  servant, CORBA_char * requirements,
+					  GNOME_stringlist * selection_order,
+					  OAF_ActivationFlags flags,
+					  OAF_ActivationCallback callback_object, 
+                                          CORBA_Context ctx,
+					  CORBA_Environment * ev)
+{
+	OAF_ActivationResult *retval = NULL;
+	OAF_ServerInfo **items, *curitem;
+	int i;
+	char *hostname;
+
+	hostname = ac_CORBA_Context_get_value (ctx, "hostname", ev);
+	ac_update_lists (servant, ev);
+
+	servant->refs++;
+
+	items = oaf_alloca (servant->total_servers *
+			    sizeof (OAF_ServerInfo *));
+	ac_query_run (servant, requirements, selection_order, ctx, items, ev);
+
+	if (ev->_major != CORBA_NO_EXCEPTION) {
+                char *message;
+                g_free (hostname);
+                servant->refs--;
+                
+                message = g_strconcat(_("Query failed: "), CORBA_exception_id (ev), NULL);
+                OAF_ActivationCallback_report_activation_failed (callback_object,
+                                                                 message, ev);
+                g_free (message);
+                return;
+        }
+
+	retval = OAF_ActivationResult__alloc ();
+	retval->res._d = OAF_RESULT_NONE;
+
+	for (i = 0; (retval->res._d == OAF_RESULT_NONE) && items[i]
+	     && (i < servant->total_servers); i++) {
+		curitem = items[i];
+
+		ac_do_activation (servant, curitem, retval, flags, hostname,
+				  ctx, ev);
+	}
+
+	if (retval->res._d == OAF_RESULT_NONE)
+		retval->aid = CORBA_string_dup ("");
+
+
+        if (ev->_major != CORBA_NO_EXCEPTION) {
+                char *message;
+                g_free (hostname);
+                CORBA_free (retval);
+                servant->refs--;
+                
+                message = g_strconcat(_("Activation failed: "), CORBA_exception_id (ev), NULL);
+                OAF_ActivationCallback_report_activation_failed (callback_object,
+                                                                 message, ev);
+                g_free (message);
+                return;
+        }
+
+        /* return the correct value back to the client */
+        OAF_ActivationCallback_report_activation_succeeded (callback_object, retval, ev);
+}
+
+
 static void
 ac_update_lists (impl_POA_OAF_ActivationContext * servant,
 		 CORBA_Environment * ev)
@@ -851,34 +929,18 @@ impl_OAF_ActivationContext_query (impl_POA_OAF_ActivationContext * servant,
 	return retval;
 }
 
-static OAF_ActivationResult *
-impl_OAF_ActivationContext_activate_from_id (impl_POA_OAF_ActivationContext *
-					     servant, OAF_ActivationID aid,
-					     OAF_ActivationFlags flags,
-					     CORBA_Context ctx,
-					     CORBA_Environment * ev)
+char *
+ac_aid_to_query_string (OAF_ActivationID aid)
 {
-	OAF_ActivationResult *retval;
-	OAFActivationInfo *ainfo;
-	char *context_username;
-	char *context_hostname;
-	char *context_domain;
         char *tmp_aid;
         char *requirements;
         char *iid_requirement;
         char *username_requirement;
         char *hostname_requirement;
         char *domain_requirement;
-        char *sort_criteria[4];
-        GNOME_stringlist selection_order;
-
-	ac_update_lists (servant, ev);
-
-	servant->refs++;
-
-	retval = OAF_ActivationResult__alloc ();
-	retval->res._d = OAF_RESULT_NONE;
+	OAFActivationInfo *ainfo;
 
+        /* FIXME: this is completely broken. we activate by AID, not IID. */
         if (strncmp ("OAFIID:", aid, 7) == 0) {
                 tmp_aid = g_strconcat ("OAFAID:[", aid, "]", NULL);
                 ainfo = oaf_actid_parse (tmp_aid);
@@ -887,8 +949,8 @@ impl_OAF_ActivationContext_activate_from_id (impl_POA_OAF_ActivationContext *
                 ainfo = oaf_actid_parse (aid);
         }
 
-	if (!ainfo) {
-                goto out;
+	if (ainfo == NULL) {
+                return NULL;
 	}
 
         iid_requirement = g_strconcat ("iid == \'", ainfo->iid, "\' ", NULL);
@@ -918,17 +980,27 @@ impl_OAF_ActivationContext_activate_from_id (impl_POA_OAF_ActivationContext *
         g_free (username_requirement);
         g_free (hostname_requirement);
         g_free (domain_requirement);
+        oaf_actinfo_free (ainfo);
 
-        /* FIXME: either I am doing something really wrong here or
-           CORBA_Context is broken in ORBit */
+        return requirements;
+}
 
-        context_username = ctx_get_value (ctx, "username", ev);
-        context_hostname = ctx_get_value (ctx, "hostname", ev);
-        context_domain = ctx_get_value (ctx, "domain", ev);
+void
+ac_context_to_string_array (CORBA_Context context, char **sort_criteria, CORBA_Environment *ev)
+{
+	char *context_username;
+	char *context_hostname;
+	char *context_domain;
 
-	if (ev->_major != CORBA_NO_EXCEPTION)
-        {
-                goto out;
+        /* FIXME bugzilla.eazel.com 2730: either I am doing something
+         * really wrong here or CORBA_Context is broken in ORBit 
+         */
+
+        context_username = ac_CORBA_Context_get_value (context, "username", ev);
+        context_hostname = ac_CORBA_Context_get_value (context, "hostname", ev);
+        context_domain = ac_CORBA_Context_get_value (context, "domain", ev);
+	if (ev->_major != CORBA_NO_EXCEPTION) {
+                return;
         }
         
         sort_criteria[0] = g_strconcat ("username == \'", context_username, "\'", NULL);
@@ -939,91 +1011,128 @@ impl_OAF_ActivationContext_activate_from_id (impl_POA_OAF_ActivationContext *
         g_free (context_username);
         g_free (context_hostname);
         g_free (context_domain);
-
-        selection_order._length = 3;
-        selection_order._buffer = sort_criteria;
-        CORBA_sequence_set_release (&selection_order, CORBA_FALSE);
-        
-        retval =
-                impl_OAF_ActivationContext_activate (servant,
-                                                     requirements,
-                                                     &selection_order,
-                                                     flags,
-                                                     ctx, ev);
-
-        g_free (sort_criteria[0]);
-        g_free (sort_criteria[1]);
-        g_free (sort_criteria[2]);
-
-        goto out;
-
-#if 0
-
-	for (cur = servant->dirs; cur && !child; cur = cur->next) {
-		ChildODInfo *curchild;
-
-		curchild = cur->data;
-
-		if (ainfo->user
-		    && (!curchild->username
-			|| strcmp (ainfo->user,
-				   curchild->username))) continue;
-		if (ainfo->host
-		    && (!curchild->hostname
-			|| strcmp (ainfo->host,
-				   curchild->hostname))) continue;
-		if (ainfo->domain
-		    && (!curchild->domain
-			|| strcmp (ainfo->domain,
-				   curchild->domain))) continue;
-		child = curchild;
-	}
-
-	if (!child)
-        {
-                CORBA_exception_set (ev, CORBA_USER_EXCEPTION,
-                                     ex_OAF_ActivationContext_IncompleteContext,
-                                     NULL);
-		goto out;	/* FIXME: in future, add hook to allow starting a new OD on demand */
-        }
-
-
-
-	if (!si)
-		goto out;
-
-	hostname = ctx_get_value (ctx, "hostname", ev);
-	if (ev->_major != CORBA_NO_EXCEPTION)
-        {
-                CORBA_exception_set (ev, CORBA_USER_EXCEPTION,
-                                     ex_OAF_ActivationContext_IncompleteContext,
-                                     NULL);
-		goto out;
-        }
-
-	ac_do_activation (servant, si, retval, flags, hostname, ctx, ev);
-
-	g_free (hostname);
-#endif
-
-      out:
-	servant->refs--;
-	if (ainfo) {
-		oaf_actinfo_free (ainfo);
-	}
-
-	if (ev->_major == CORBA_NO_EXCEPTION) {
-		if (retval->res._d == OAF_RESULT_NONE)
-			retval->aid = CORBA_string_dup ("");
-	} else {
-		CORBA_free (retval);
-		retval = NULL;
-	}
-
-	return retval;
 }
 
 
 
 
+static OAF_ActivationResult *
+impl_OAF_ActivationContext_activate_from_id (impl_POA_OAF_ActivationContext *
+					     servant, OAF_ActivationID aid,
+					     OAF_ActivationFlags flags,
+					     CORBA_Context ctx,
+					     CORBA_Environment * ev)
+{
+	OAF_ActivationResult *retval;
+        char *requirements;
+        char *sort_criteria[4];
+        GNOME_stringlist selection_order;
+
+	ac_update_lists (servant, ev);
+        if (ev->_major != CORBA_NO_EXCEPTION) {
+                return CORBA_OBJECT_NIL;
+        }
+
+
+	servant->refs++;
+
+        requirements = ac_aid_to_query_string (aid);
+        if (requirements == NULL) {
+                servant->refs--;
+                return CORBA_OBJECT_NIL;
+        }
+
+        ac_context_to_string_array (ctx, sort_criteria, ev);
+        if (ev->_major != CORBA_NO_EXCEPTION) {
+                servant->refs--;
+                return CORBA_OBJECT_NIL;
+        }
+
+        selection_order._length = 3;
+        selection_order._buffer = sort_criteria;
+        CORBA_sequence_set_release (&selection_order, CORBA_FALSE);
+
+        retval = impl_OAF_ActivationContext_activate (servant,
+                                                      requirements,
+                                                      &selection_order,
+                                                      flags,
+                                                      ctx, ev);        
+        g_free (sort_criteria[0]);
+        g_free (sort_criteria[1]);
+        g_free (sort_criteria[2]);
+
+        servant->refs--;
+
+        return retval;
+}
+
+
+static void
+impl_OAF_ActivationContext_activate_from_id_async
+                (impl_POA_OAF_ActivationContext * servant, 
+                 OAF_ActivationID aid,
+                 OAF_ActivationFlags flags, 
+                 OAF_ActivationCallback callback_object,
+                 CORBA_Context ctx, 
+                 CORBA_Environment * ev)
+{
+	OAF_ActivationResult *retval;
+        char *requirements;
+        char *sort_criteria[4];
+        GNOME_stringlist selection_order;
+
+	ac_update_lists (servant, ev);
+
+	servant->refs++;
+
+        requirements = ac_aid_to_query_string (aid);
+        if (requirements == NULL) {
+                servant->refs--;
+                OAF_ActivationCallback_report_activation_failed (callback_object,
+                                                                 _("Could not parse AID"), ev);
+                return;
+        }
+
+
+        ac_context_to_string_array (ctx, sort_criteria, ev);
+        if (ev->_major != CORBA_NO_EXCEPTION) {
+                char *message;
+                servant->refs--;
+                message = g_strconcat (_("Could not parse context: "), CORBA_exception_id (ev), NULL);
+                OAF_ActivationCallback_report_activation_failed (callback_object,
+                                                                 message, ev);
+                g_free (message);
+                return;
+        }
+
+        selection_order._length = 3;
+        selection_order._buffer = sort_criteria;
+        CORBA_sequence_set_release (&selection_order, CORBA_FALSE);
+
+        retval = impl_OAF_ActivationContext_activate (servant,
+                                                      requirements,
+                                                      &selection_order,
+                                                      flags,
+                                                      ctx, ev);
+        g_free (sort_criteria[0]);
+        g_free (sort_criteria[1]);
+        g_free (sort_criteria[2]);
+
+        if (ev->_major != CORBA_NO_EXCEPTION) {
+                char *message;
+                servant->refs--;
+                message = g_strconcat (_("Could not activate server: "), CORBA_exception_id (ev), NULL);
+                OAF_ActivationCallback_report_activation_failed (callback_object,
+                                                                 message, ev);
+                g_free (message);
+                return;
+        }
+
+        servant->refs--;
+
+        OAF_ActivationCallback_report_activation_succeeded (callback_object,
+                                                            retval, ev);
+}
+
+
 
